# coding: utf-8

from __future__ import print_function

import pytest  # NOQA

from .roundtrip import round_trip, dedent, round_trip_load, round_trip_dump  # NOQA


def load(s):
    return round_trip_load(dedent(s))


def compare(data, s, **kw):
    assert round_trip_dump(data, **kw) == dedent(s)


def compare_eol(data, s):
    assert "EOL" in s
    ds = dedent(s).replace("EOL", "").replace("\n", "|\n")
    assert round_trip_dump(data).replace("\n", "|\n") == ds


class TestCommentsManipulation:

    # list
    def test_seq_set_comment_on_existing_explicit_column(self):
        data = load(
            """
        - a   # comment 1
        - b
        - c
        """
        )
        data.yaml_add_eol_comment("comment 2", key=1, column=6)
        exp = """
        - a   # comment 1
        - b   # comment 2
        - c
        """
        compare(data, exp)

    def test_seq_overwrite_comment_on_existing_explicit_column(self):
        data = load(
            """
        - a   # comment 1
        - b
        - c
        """
        )
        data.yaml_add_eol_comment("comment 2", key=0, column=6)
        exp = """
        - a   # comment 2
        - b
        - c
        """
        compare(data, exp)

    def test_seq_first_comment_explicit_column(self):
        data = load(
            """
        - a
        - b
        - c
        """
        )
        data.yaml_add_eol_comment("comment 1", key=1, column=6)
        exp = """
        - a
        - b   # comment 1
        - c
        """
        compare(data, exp)

    def test_seq_set_comment_on_existing_column_prev(self):
        data = load(
            """
        - a   # comment 1
        - b
        - c
        - d     # comment 3
        """
        )
        data.yaml_add_eol_comment("comment 2", key=1)
        exp = """
        - a   # comment 1
        - b   # comment 2
        - c
        - d     # comment 3
        """
        compare(data, exp)

    def test_seq_set_comment_on_existing_column_next(self):
        data = load(
            """
        - a   # comment 1
        - b
        - c
        - d     # comment 3
        """
        )
        print(data._yaml_comment)
        # print(type(data._yaml_comment._items[0][0].start_mark))
        # srsly.ruamel_yaml.error.Mark
        # print(type(data._yaml_comment._items[0][0].start_mark))
        data.yaml_add_eol_comment("comment 2", key=2)
        exp = """
        - a   # comment 1
        - b
        - c     # comment 2
        - d     # comment 3
        """
        compare(data, exp)

    def test_seq_set_comment_on_existing_column_further_away(self):
        """
        no comment line before or after, take the latest before
        the new position
        """
        data = load(
            """
        - a   # comment 1
        - b
        - c
        - d
        - e
        - f     # comment 3
        """
        )
        print(data._yaml_comment)
        # print(type(data._yaml_comment._items[0][0].start_mark))
        # srsly.ruamel_yaml.error.Mark
        # print(type(data._yaml_comment._items[0][0].start_mark))
        data.yaml_add_eol_comment("comment 2", key=3)
        exp = """
        - a   # comment 1
        - b
        - c
        - d   # comment 2
        - e
        - f     # comment 3
        """
        compare(data, exp)

    def test_seq_set_comment_on_existing_explicit_column_with_hash(self):
        data = load(
            """
        - a   # comment 1
        - b
        - c
        """
        )
        data.yaml_add_eol_comment("#  comment 2", key=1, column=6)
        exp = """
        - a   # comment 1
        - b   #  comment 2
        - c
        """
        compare(data, exp)

    # dict

    def test_dict_set_comment_on_existing_explicit_column(self):
        data = load(
            """
        a: 1   # comment 1
        b: 2
        c: 3
        d: 4
        e: 5
        """
        )
        data.yaml_add_eol_comment("comment 2", key="c", column=7)
        exp = """
        a: 1   # comment 1
        b: 2
        c: 3   # comment 2
        d: 4
        e: 5
        """
        compare(data, exp)

    def test_dict_overwrite_comment_on_existing_explicit_column(self):
        data = load(
            """
        a: 1   # comment 1
        b: 2
        c: 3
        d: 4
        e: 5
        """
        )
        data.yaml_add_eol_comment("comment 2", key="a", column=7)
        exp = """
        a: 1   # comment 2
        b: 2
        c: 3
        d: 4
        e: 5
        """
        compare(data, exp)

    def test_map_set_comment_on_existing_column_prev(self):
        data = load(
            """
            a: 1   # comment 1
            b: 2
            c: 3
            d: 4
            e: 5     # comment 3
            """
        )
        data.yaml_add_eol_comment("comment 2", key="b")
        exp = """
            a: 1   # comment 1
            b: 2   # comment 2
            c: 3
            d: 4
            e: 5     # comment 3
            """
        compare(data, exp)

    def test_map_set_comment_on_existing_column_next(self):
        data = load(
            """
            a: 1   # comment 1
            b: 2
            c: 3
            d: 4
            e: 5     # comment 3
            """
        )
        data.yaml_add_eol_comment("comment 2", key="d")
        exp = """
            a: 1   # comment 1
            b: 2
            c: 3
            d: 4     # comment 2
            e: 5     # comment 3
            """
        compare(data, exp)

    def test_map_set_comment_on_existing_column_further_away(self):
        """
        no comment line before or after, take the latest before
        the new position
        """
        data = load(
            """
            a: 1   # comment 1
            b: 2
            c: 3
            d: 4
            e: 5     # comment 3
            """
        )
        data.yaml_add_eol_comment("comment 2", key="c")
        print(round_trip_dump(data))
        exp = """
            a: 1   # comment 1
            b: 2
            c: 3   # comment 2
            d: 4
            e: 5     # comment 3
            """
        compare(data, exp)

    def test_before_top_map_rt(self):
        data = load(
            """
        a: 1
        b: 2
        """
        )
        data.yaml_set_start_comment("Hello\nWorld\n")
        exp = """
        # Hello
        # World
        a: 1
        b: 2
        """
        compare(data, exp.format(comment="#"))

    def test_before_top_map_replace(self):
        data = load(
            """
        # abc
        # def
        a: 1 # 1
        b: 2
        """
        )
        data.yaml_set_start_comment("Hello\nWorld\n")
        exp = """
        # Hello
        # World
        a: 1 # 1
        b: 2
        """
        compare(data, exp.format(comment="#"))

    def test_before_top_map_from_scratch(self):
        from srsly.ruamel_yaml.comments import CommentedMap

        data = CommentedMap()
        data["a"] = 1
        data["b"] = 2
        data.yaml_set_start_comment("Hello\nWorld\n")
        # print(data.ca)
        # print(data.ca._items)
        exp = """
            # Hello
            # World
            a: 1
            b: 2
            """
        compare(data, exp.format(comment="#"))

    def test_before_top_seq_rt(self):
        data = load(
            """
        - a
        - b
        """
        )
        data.yaml_set_start_comment("Hello\nWorld\n")
        print(round_trip_dump(data))
        exp = """
        # Hello
        # World
        - a
        - b
        """
        compare(data, exp)

    def test_before_top_seq_rt_replace(self):
        s = """
        # this
        # that
        - a
        - b
        """
        data = load(s.format(comment="#"))
        data.yaml_set_start_comment("Hello\nWorld\n")
        print(round_trip_dump(data))
        exp = """
        # Hello
        # World
        - a
        - b
        """
        compare(data, exp.format(comment="#"))

    def test_before_top_seq_from_scratch(self):
        from srsly.ruamel_yaml.comments import CommentedSeq

        data = CommentedSeq()
        data.append("a")
        data.append("b")
        data.yaml_set_start_comment("Hello\nWorld\n")
        print(round_trip_dump(data))
        exp = """
        # Hello
        # World
        - a
        - b
        """
        compare(data, exp.format(comment="#"))

    # nested variants
    def test_before_nested_map_rt(self):
        data = load(
            """
        a: 1
        b:
          c: 2
          d: 3
        """
        )
        data["b"].yaml_set_start_comment("Hello\nWorld\n")
        exp = """
        a: 1
        b:
        # Hello
        # World
          c: 2
          d: 3
        """
        compare(data, exp.format(comment="#"))

    def test_before_nested_map_rt_indent(self):
        data = load(
            """
        a: 1
        b:
          c: 2
          d: 3
        """
        )
        data["b"].yaml_set_start_comment("Hello\nWorld\n", indent=2)
        exp = """
        a: 1
        b:
          # Hello
          # World
          c: 2
          d: 3
        """
        compare(data, exp.format(comment="#"))
        print(data["b"].ca)

    def test_before_nested_map_from_scratch(self):
        from srsly.ruamel_yaml.comments import CommentedMap

        data = CommentedMap()
        datab = CommentedMap()
        data["a"] = 1
        data["b"] = datab
        datab["c"] = 2
        datab["d"] = 3
        data["b"].yaml_set_start_comment("Hello\nWorld\n")
        exp = """
        a: 1
        b:
        # Hello
        # World
          c: 2
          d: 3
        """
        compare(data, exp.format(comment="#"))

    def test_before_nested_seq_from_scratch(self):
        from srsly.ruamel_yaml.comments import CommentedMap, CommentedSeq

        data = CommentedMap()
        datab = CommentedSeq()
        data["a"] = 1
        data["b"] = datab
        datab.append("c")
        datab.append("d")
        data["b"].yaml_set_start_comment("Hello\nWorld\n", indent=2)
        exp = """
        a: 1
        b:
          # Hello
          # World
        - c
        - d
        """
        compare(data, exp.format(comment="#"))

    def test_before_nested_seq_from_scratch_block_seq_indent(self):
        from srsly.ruamel_yaml.comments import CommentedMap, CommentedSeq

        data = CommentedMap()
        datab = CommentedSeq()
        data["a"] = 1
        data["b"] = datab
        datab.append("c")
        datab.append("d")
        data["b"].yaml_set_start_comment("Hello\nWorld\n", indent=2)
        exp = """
        a: 1
        b:
          # Hello
          # World
          - c
          - d
        """
        compare(data, exp.format(comment="#"), indent=4, block_seq_indent=2)

    def test_map_set_comment_before_and_after_non_first_key_00(self):
        # http://stackoverflow.com/a/40705671/1307905
        data = load(
            """
        xyz:
          a: 1    # comment 1
          b: 2

        test1:
          test2:
            test3: 3
                """
        )
        data.yaml_set_comment_before_after_key(
            "test1", "before test1 (top level)", after="before test2"
        )
        data["test1"]["test2"].yaml_set_start_comment("after test2", indent=4)
        exp = """
        xyz:
          a: 1    # comment 1
          b: 2

        # before test1 (top level)
        test1:
          # before test2
          test2:
            # after test2
            test3: 3
        """
        compare(data, exp)

    def Xtest_map_set_comment_before_and_after_non_first_key_01(self):
        data = load(
            """
        xyz:
          a: 1    # comment 1
          b: 2

        test1:
          test2:
            test3: 3
        """
        )
        data.yaml_set_comment_before_after_key(
            "test1", "before test1 (top level)", after="before test2\n\n"
        )
        data["test1"]["test2"].yaml_set_start_comment("after test2", indent=4)
        # EOL is needed here as dedenting gets rid of spaces (as well as does Emacs
        exp = """
        xyz:
          a: 1    # comment 1
          b: 2

        # before test1 (top level)
        test1:
          # before test2
          EOL
          test2:
            # after test2
            test3: 3
        """
        compare_eol(data, exp)

    # EOL is no longer necessary
    # fixed together with issue # 216
    def test_map_set_comment_before_and_after_non_first_key_01(self):
        data = load(
            """
        xyz:
          a: 1    # comment 1
          b: 2

        test1:
          test2:
            test3: 3
        """
        )
        data.yaml_set_comment_before_after_key(
            "test1", "before test1 (top level)", after="before test2\n\n"
        )
        data["test1"]["test2"].yaml_set_start_comment("after test2", indent=4)
        exp = """
        xyz:
          a: 1    # comment 1
          b: 2

        # before test1 (top level)
        test1:
          # before test2

          test2:
            # after test2
            test3: 3
        """
        compare(data, exp)

    def Xtest_map_set_comment_before_and_after_non_first_key_02(self):
        data = load(
            """
        xyz:
          a: 1    # comment 1
          b: 2

        test1:
          test2:
            test3: 3
        """
        )
        data.yaml_set_comment_before_after_key(
            "test1",
            "xyz\n\nbefore test1 (top level)",
            after="\nbefore test2",
            after_indent=4,
        )
        data["test1"]["test2"].yaml_set_start_comment("after test2", indent=4)
        # EOL is needed here as dedenting gets rid of spaces (as well as does Emacs
        exp = """
        xyz:
          a: 1    # comment 1
          b: 2

        # xyz

        # before test1 (top level)
        test1:
            EOL
            # before test2
          test2:
            # after test2
            test3: 3
        """
        compare_eol(data, exp)

    def test_map_set_comment_before_and_after_non_first_key_02(self):
        data = load(
            """
        xyz:
          a: 1    # comment 1
          b: 2

        test1:
          test2:
            test3: 3
        """
        )
        data.yaml_set_comment_before_after_key(
            "test1",
            "xyz\n\nbefore test1 (top level)",
            after="\nbefore test2",
            after_indent=4,
        )
        data["test1"]["test2"].yaml_set_start_comment("after test2", indent=4)
        exp = """
        xyz:
          a: 1    # comment 1
          b: 2

        # xyz

        # before test1 (top level)
        test1:

            # before test2
          test2:
            # after test2
            test3: 3
        """
        compare(data, exp)



# coding: utf-8

"""
comment testing is all about roundtrips
these can be done in the "old" way by creating a file.data and file.roundtrip
but there is little flexibility in doing that

but some things are not easily tested, eog. how a
roundtrip changes

"""

import pytest
import sys

from .roundtrip import round_trip, dedent, round_trip_load, round_trip_dump


class TestComments:
    def test_no_end_of_file_eol(self):
        """not excluding comments caused some problems if at the end of
        the file without a newline. First error, then included \0 """
        x = """\
        - europe: 10 # abc"""
        round_trip(x, extra="\n")
        with pytest.raises(AssertionError):
            round_trip(x, extra="a\n")

    def test_no_comments(self):
        round_trip(
            """
        - europe: 10
        - usa:
          - ohio: 2
          - california: 9
        """
        )

    def test_round_trip_ordering(self):
        round_trip(
            """
        a: 1
        b: 2
        c: 3
        b1: 2
        b2: 2
        d: 4
        e: 5
        f: 6
        """
        )

    def test_complex(self):
        round_trip(
            """
        - europe: 10 # top
        - usa:
          - ohio: 2
          - california: 9 # o
        """
        )

    def test_dropped(self):
        s = """\
        # comment
        scalar
        ...
        """
        round_trip(s, "scalar\n...\n")

    def test_main_mapping_begin_end(self):
        round_trip(
            """
        # C start a
        # C start b
        abc: 1
        ghi: 2
        klm: 3
        # C end a
        # C end b
        """
        )

    def test_reindent(self):
        x = """\
        a:
          b:     # comment 1
            c: 1 # comment 2
        """
        d = round_trip_load(x)
        y = round_trip_dump(d, indent=4)
        assert y == dedent(
            """\
        a:
            b:   # comment 1
                c: 1 # comment 2
        """
        )

    def test_main_mapping_begin_end_items_post(self):
        round_trip(
            """
        # C start a
        # C start b
        abc: 1      # abc comment
        ghi: 2
        klm: 3      # klm comment
        # C end a
        # C end b
        """
        )

    def test_main_sequence_begin_end(self):
        round_trip(
            """
        # C start a
        # C start b
        - abc
        - ghi
        - klm
        # C end a
        # C end b
        """
        )

    def test_main_sequence_begin_end_items_post(self):
        round_trip(
            """
        # C start a
        # C start b
        - abc      # abc comment
        - ghi
        - klm      # klm comment
        # C end a
        # C end b
        """
        )

    def test_main_mapping_begin_end_complex(self):
        round_trip(
            """
        # C start a
        # C start b
        abc: 1
        ghi: 2
        klm:
          3a: alpha
          3b: beta   # it is all greek to me
        # C end a
        # C end b
        """
        )

    def test_09(self):  # 2.9 from the examples in the spec
        s = """\
        hr: # 1998 hr ranking
          - Mark McGwire
          - Sammy Sosa
        rbi:
          # 1998 rbi ranking
          - Sammy Sosa
          - Ken Griffey
        """
        round_trip(s, indent=4, block_seq_indent=2)

    def test_09a(self):
        round_trip(
            """
        hr: # 1998 hr ranking
        - Mark McGwire
        - Sammy Sosa
        rbi:
          # 1998 rbi ranking
        - Sammy Sosa
        - Ken Griffey
        """
        )

    def test_simple_map_middle_comment(self):
        round_trip(
            """
        abc: 1
        # C 3a
        # C 3b
        ghi: 2
        """
        )

    def test_map_in_map_0(self):
        round_trip(
            """
        map1: # comment 1
          # comment 2
          map2:
            key1: val1
        """
        )

    def test_map_in_map_1(self):
        # comment is moved from value to key
        round_trip(
            """
        map1:
          # comment 1
          map2:
            key1: val1
        """
        )

    def test_application_arguments(self):
        # application configur
        round_trip(
            """
        args:
          username: anthon
          passwd: secret
          fullname: Anthon van der Neut
          tmux:
            session-name: test
          loop:
            wait: 10
        """
        )

    def test_substitute(self):
        x = """
        args:
          username: anthon          # name
          passwd: secret            # password
          fullname: Anthon van der Neut
          tmux:
            session-name: test
          loop:
            wait: 10
        """
        data = round_trip_load(x)
        data["args"]["passwd"] = "deleted password"
        # note the requirement to add spaces for alignment of comment
        x = x.replace(": secret          ", ": deleted password")
        assert round_trip_dump(data) == dedent(x)

    def test_set_comment(self):
        round_trip(
            """
        !!set
        # the beginning
        ? a
        # next one is B (lowercase)
        ? b  #  You see? Promised you.
        ? c
        # this is the end
        """
        )

    def test_omap_comment_roundtrip(self):
        round_trip(
            """
        !!omap
        - a: 1
        - b: 2  # two
        - c: 3  # three
        - d: 4
        """
        )

    def test_omap_comment_roundtrip_pre_comment(self):
        round_trip(
            """
        !!omap
        - a: 1
        - b: 2  # two
        - c: 3  # three
        # last one
        - d: 4
        """
        )

    def test_non_ascii(self):
        round_trip(
            """
        verbosity: 1                  # 0 is minimal output, -1 none
        base_url: http://gopher.net
        special_indices: [1, 5, 8]
        also_special:
        - a
        - 19
        - 32
        asia and europe: &asia_europe
          Turkey: Ankara
          Russia: Moscow
        countries:
          Asia:
            <<: *asia_europe
            Japan: Tokyo # 東京
          Europe:
            <<: *asia_europe
            Spain: Madrid
            Italy: Rome
        """
        )

    def test_dump_utf8(self):
        import srsly.ruamel_yaml  # NOQA

        x = dedent(
            """\
        ab:
        - x  # comment
        - y  # more comment
        """
        )
        data = round_trip_load(x)
        dumper = srsly.ruamel_yaml.RoundTripDumper
        for utf in [True, False]:
            y = srsly.ruamel_yaml.dump(
                data, default_flow_style=False, Dumper=dumper, allow_unicode=utf
            )
            assert y == x

    def test_dump_unicode_utf8(self):
        import srsly.ruamel_yaml  # NOQA

        x = dedent(
            u"""\
        ab:
        - x  # comment
        - y  # more comment
        """
        )
        data = round_trip_load(x)
        dumper = srsly.ruamel_yaml.RoundTripDumper
        for utf in [True, False]:
            y = srsly.ruamel_yaml.dump(
                data, default_flow_style=False, Dumper=dumper, allow_unicode=utf
            )
            assert y == x

    def test_mlget_00(self):
        x = """\
        a:
        - b:
          c: 42
        - d:
            f: 196
          e:
            g: 3.14
        """
        d = round_trip_load(x)
        assert d.mlget(["a", 1, "d", "f"], list_ok=True) == 196
        with pytest.raises(AssertionError):
            d.mlget(["a", 1, "d", "f"]) == 196


class TestInsertPopList:
    """list insertion is more complex than dict insertion, as you
    need to move the values to subsequent keys on insert"""

    @property
    def ins(self):
        return """\
        ab:
        - a      # a
        - b      # b
        - c
        - d      # d

        de:
        - 1
        - 2
        """

    def test_insert_0(self):
        d = round_trip_load(self.ins)
        d["ab"].insert(0, "xyz")
        y = round_trip_dump(d, indent=2)
        assert y == dedent(
            """\
        ab:
        - xyz
        - a      # a
        - b      # b
        - c
        - d      # d

        de:
        - 1
        - 2
        """
        )

    def test_insert_1(self):
        d = round_trip_load(self.ins)
        d["ab"].insert(4, "xyz")
        y = round_trip_dump(d, indent=2)
        assert y == dedent(
            """\
        ab:
        - a      # a
        - b      # b
        - c
        - d      # d

        - xyz
        de:
        - 1
        - 2
        """
        )

    def test_insert_2(self):
        d = round_trip_load(self.ins)
        d["ab"].insert(1, "xyz")
        y = round_trip_dump(d, indent=2)
        assert y == dedent(
            """\
        ab:
        - a      # a
        - xyz
        - b      # b
        - c
        - d      # d

        de:
        - 1
        - 2
        """
        )

    def test_pop_0(self):
        d = round_trip_load(self.ins)
        d["ab"].pop(0)
        y = round_trip_dump(d, indent=2)
        print(y)
        assert y == dedent(
            """\
        ab:
        - b      # b
        - c
        - d      # d

        de:
        - 1
        - 2
        """
        )

    def test_pop_1(self):
        d = round_trip_load(self.ins)
        d["ab"].pop(1)
        y = round_trip_dump(d, indent=2)
        print(y)
        assert y == dedent(
            """\
        ab:
        - a      # a
        - c
        - d      # d

        de:
        - 1
        - 2
        """
        )

    def test_pop_2(self):
        d = round_trip_load(self.ins)
        d["ab"].pop(2)
        y = round_trip_dump(d, indent=2)
        print(y)
        assert y == dedent(
            """\
        ab:
        - a      # a
        - b      # b
        - d      # d

        de:
        - 1
        - 2
        """
        )

    def test_pop_3(self):
        d = round_trip_load(self.ins)
        d["ab"].pop(3)
        y = round_trip_dump(d, indent=2)
        print(y)
        assert y == dedent(
            """\
        ab:
        - a      # a
        - b      # b
        - c
        de:
        - 1
        - 2
        """
        )


# inspired by demux' question on stackoverflow
# http://stackoverflow.com/a/36970608/1307905
class TestInsertInMapping:
    @property
    def ins(self):
        return """\
        first_name: Art
        occupation: Architect  # This is an occupation comment
        about: Art Vandelay is a fictional character that George invents...
        """

    def test_insert_at_pos_1(self):
        d = round_trip_load(self.ins)
        d.insert(1, "last name", "Vandelay", comment="new key")
        y = round_trip_dump(d)
        print(y)
        assert y == dedent(
            """\
        first_name: Art
        last name: Vandelay    # new key
        occupation: Architect  # This is an occupation comment
        about: Art Vandelay is a fictional character that George invents...
        """
        )

    def test_insert_at_pos_0(self):
        d = round_trip_load(self.ins)
        d.insert(0, "last name", "Vandelay", comment="new key")
        y = round_trip_dump(d)
        print(y)
        assert y == dedent(
            """\
        last name: Vandelay  # new key
        first_name: Art
        occupation: Architect  # This is an occupation comment
        about: Art Vandelay is a fictional character that George invents...
        """
        )

    def test_insert_at_pos_3(self):
        # much more simple if done with appending.
        d = round_trip_load(self.ins)
        d.insert(3, "last name", "Vandelay", comment="new key")
        y = round_trip_dump(d)
        print(y)
        assert y == dedent(
            """\
        first_name: Art
        occupation: Architect  # This is an occupation comment
        about: Art Vandelay is a fictional character that George invents...
        last name: Vandelay  # new key
        """
        )


class TestCommentedMapMerge:
    def test_in_operator(self):
        data = round_trip_load(
            """
        x: &base
          a: 1
          b: 2
          c: 3
        y:
          <<: *base
          k: 4
          l: 5
        """
        )
        assert data["x"]["a"] == 1
        assert "a" in data["x"]
        assert data["y"]["a"] == 1
        assert "a" in data["y"]

    def test_issue_60(self):
        data = round_trip_load(
            """
        x: &base
          a: 1
        y:
          <<: *base
        """
        )
        assert data["x"]["a"] == 1
        assert data["y"]["a"] == 1
        assert str(data["y"]) == """ordereddict([('a', 1)])"""

    def test_issue_60_1(self):
        data = round_trip_load(
            """
        x: &base
          a: 1
        y:
          <<: *base
          b: 2
        """
        )
        assert data["x"]["a"] == 1
        assert data["y"]["a"] == 1
        assert str(data["y"]) == """ordereddict([('b', 2), ('a', 1)])"""


class TestEmptyLines:
    # prompted by issue 46 from Alex Harvey
    def test_issue_46(self):
        yaml_str = dedent(
            """\
        ---
        # Please add key/value pairs in alphabetical order

        aws_s3_bucket: 'mys3bucket'

        jenkins_ad_credentials:
          bind_name: 'CN=svc-AAA-BBB-T,OU=Example,DC=COM,DC=EXAMPLE,DC=Local'
          bind_pass: 'xxxxyyyy{'
        """
        )
        d = round_trip_load(yaml_str, preserve_quotes=True)
        y = round_trip_dump(d, explicit_start=True)
        assert yaml_str == y

    def test_multispace_map(self):
        round_trip(
            """
        a: 1x

        b: 2x


        c: 3x



        d: 4x

        """
        )

    @pytest.mark.xfail(strict=True)
    def test_multispace_map_initial(self):
        round_trip(
            """

        a: 1x

        b: 2x


        c: 3x



        d: 4x

        """
        )

    def test_embedded_map(self):
        round_trip(
            """
        - a: 1y
          b: 2y

          c: 3y
        """
        )

    def test_toplevel_seq(self):
        round_trip(
            """\
        - 1

        - 2

        - 3
        """
        )

    def test_embedded_seq(self):
        round_trip(
            """
        a:
          b:
          - 1

          - 2


          - 3
        """
        )

    def test_line_with_only_spaces(self):
        # issue 54
        yaml_str = "---\n\na: 'x'\n \nb: y\n"
        d = round_trip_load(yaml_str, preserve_quotes=True)
        y = round_trip_dump(d, explicit_start=True)
        stripped = ""
        for line in yaml_str.splitlines():
            stripped += line.rstrip() + "\n"
            print(line + "$")
        assert stripped == y

    def test_some_eol_spaces(self):
        # spaces after tokens and on empty lines
        yaml_str = '---  \n  \na: "x"  \n   \nb: y  \n'
        d = round_trip_load(yaml_str, preserve_quotes=True)
        y = round_trip_dump(d, explicit_start=True)
        stripped = ""
        for line in yaml_str.splitlines():
            stripped += line.rstrip() + "\n"
            print(line + "$")
        assert stripped == y

    def test_issue_54_not_ok(self):
        yaml_str = dedent(
            """\
        toplevel:

            # some comment
            sublevel: 300
        """
        )
        d = round_trip_load(yaml_str)
        print(d.ca)
        y = round_trip_dump(d, indent=4)
        print(y.replace("\n", "$\n"))
        assert yaml_str == y

    def test_issue_54_ok(self):
        yaml_str = dedent(
            """\
        toplevel:
            # some comment
            sublevel: 300
        """
        )
        d = round_trip_load(yaml_str)
        y = round_trip_dump(d, indent=4)
        assert yaml_str == y

    def test_issue_93(self):
        round_trip(
            """\
        a:
          b:
          - c1: cat  # a1
          # my comment on catfish
          - c2: catfish  # a2
        """
        )

    def test_issue_93_00(self):
        round_trip(
            """\
        a:
        - - c1: cat   # a1
          # my comment on catfish
          - c2: catfish  # a2
        """
        )

    def test_issue_93_01(self):
        round_trip(
            """\
        - - c1: cat   # a1
          # my comment on catfish
          - c2: catfish  # a2
        """
        )

    def test_issue_93_02(self):
        # never failed as there is no indent
        round_trip(
            """\
        - c1: cat
        # my comment on catfish
        - c2: catfish
        """
        )

    def test_issue_96(self):
        # inserted extra line on trailing spaces
        round_trip(
            """\
        a:
          b:
            c: c_val
            d:

          e:
            g: g_val
        """
        )


class TestUnicodeComments:
    @pytest.mark.skipif(sys.version_info < (2, 7), reason="wide unicode")
    def test_issue_55(self):  # reported by Haraguroicha Hsu
        round_trip(
            """\
        name: TEST
        description: test using
        author: Harguroicha
        sql:
          command: |-
            select name from testtbl where no = :no

          ci-test:
          - :no: 04043709 # 小花
          - :no: 05161690 # 茶
          - :no: 05293147 # 〇𤋥川
          - :no: 05338777 # 〇〇啓
          - :no: 05273867 # 〇
          - :no: 05205786 # 〇𤦌
        """
        )


class TestEmptyValueBeforeComments:
    def test_issue_25a(self):
        round_trip(
            """\
        - a: b
          c: d
          d:  # foo
          - e: f
        """
        )

    def test_issue_25a1(self):
        round_trip(
            """\
        - a: b
          c: d
          d:  # foo
            e: f
        """
        )

    def test_issue_25b(self):
        round_trip(
            """\
        var1: #empty
        var2: something #notempty
        """
        )

    def test_issue_25c(self):
        round_trip(
            """\
        params:
          a: 1 # comment a
          b:   # comment b
          c: 3 # comment c
        """
        )

    def test_issue_25c1(self):
        round_trip(
            """\
        params:
          a: 1 # comment a
          b:   # comment b
          # extra
          c: 3 # comment c
        """
        )

    def test_issue_25_00(self):
        round_trip(
            """\
        params:
          a: 1 # comment a
          b:   # comment b
        """
        )

    def test_issue_25_01(self):
        round_trip(
            """\
        a:        # comment 1
                  #  comment 2
        - b:      #   comment 3
            c: 1  #    comment 4
        """
        )

    def test_issue_25_02(self):
        round_trip(
            """\
        a:        # comment 1
                  #  comment 2
        - b: 2    #   comment 3
        """
        )

    def test_issue_25_03(self):
        s = """\
        a:        # comment 1
                  #  comment 2
          - b: 2  #   comment 3
        """
        round_trip(s, indent=4, block_seq_indent=2)

    def test_issue_25_04(self):
        round_trip(
            """\
        a:        # comment 1
                  #  comment 2
          b: 1    #   comment 3
        """
        )

    def test_flow_seq_within_seq(self):
        round_trip(
            """\
        # comment 1
        - a
        - b
        # comment 2
        - c
        - d
        # comment 3
        - [e]
        - f
        # comment 4
        - []
        """
        )


test_block_scalar_commented_line_template = """\
y: p
# Some comment

a: |
  x
{}b: y
"""


class TestBlockScalarWithComments:
    # issue 99 reported by Colm O'Connor
    def test_scalar_with_comments(self):
        import srsly.ruamel_yaml  # NOQA

        for x in [
            "",
            "\n",
            "\n# Another comment\n",
            "\n\n",
            "\n\n# abc\n#xyz\n",
            "\n\n# abc\n#xyz\n",
            "# abc\n\n#xyz\n",
            "\n\n  # abc\n  #xyz\n",
        ]:

            commented_line = test_block_scalar_commented_line_template.format(x)
            data = srsly.ruamel_yaml.round_trip_load(commented_line)

            assert srsly.ruamel_yaml.round_trip_dump(data) == commented_line



# coding: utf-8

"""
test flow style sequences as keys roundtrip

"""

# import pytest

from .roundtrip import round_trip  # , dedent, round_trip_load, round_trip_dump


class TestFlowStyleSequenceKey:
    def test_so_39595807(self):
        inp = """\
        %YAML 1.2
        ---
        [2, 3, 4]:
          a:
          - 1
          - 2
          b: Hello World!
          c: 'Voilà!'
        """
        round_trip(inp, preserve_quotes=True, explicit_start=True, version=(1, 2))



# coding: utf-8

from __future__ import print_function

"""
testing of anchors and the aliases referring to them
"""

import sys
import textwrap
import pytest
from pathlib import Path


class TestNewAPI:
    def test_duplicate_keys_00(self):
        from srsly.ruamel_yaml import YAML
        from srsly.ruamel_yaml.constructor import DuplicateKeyError

        yaml = YAML()
        with pytest.raises(DuplicateKeyError):
            yaml.load("{a: 1, a: 2}")

    def test_duplicate_keys_01(self):
        from srsly.ruamel_yaml import YAML
        from srsly.ruamel_yaml.constructor import DuplicateKeyError

        yaml = YAML(typ="safe", pure=True)
        with pytest.raises(DuplicateKeyError):
            yaml.load("{a: 1, a: 2}")

    def test_duplicate_keys_02(self):
        from srsly.ruamel_yaml import YAML
        from srsly.ruamel_yaml.constructor import DuplicateKeyError

        yaml = YAML(typ="safe")
        with pytest.raises(DuplicateKeyError):
            yaml.load("{a: 1, a: 2}")

    def test_issue_135(self):
        # reported by Andrzej Ostrowski
        from srsly.ruamel_yaml import YAML

        data = {"a": 1, "b": 2}
        yaml = YAML(typ="safe")
        # originally on 2.7: with pytest.raises(TypeError):
        yaml.dump(data, sys.stdout)

    def test_issue_135_temporary_workaround(self):
        # never raised error
        from srsly.ruamel_yaml import YAML

        data = {"a": 1, "b": 2}
        yaml = YAML(typ="safe", pure=True)
        yaml.dump(data, sys.stdout)


class TestWrite:
    def test_dump_path(self, tmpdir):
        from srsly.ruamel_yaml import YAML

        fn = Path(str(tmpdir)) / "test.yaml"
        yaml = YAML()
        data = yaml.map()
        data["a"] = 1
        data["b"] = 2
        yaml.dump(data, fn)
        assert fn.read_text() == "a: 1\nb: 2\n"

    def test_dump_file(self, tmpdir):
        from srsly.ruamel_yaml import YAML

        fn = Path(str(tmpdir)) / "test.yaml"
        yaml = YAML()
        data = yaml.map()
        data["a"] = 1
        data["b"] = 2
        with open(str(fn), "w") as fp:
            yaml.dump(data, fp)
        assert fn.read_text() == "a: 1\nb: 2\n"

    def test_dump_missing_stream(self):
        from srsly.ruamel_yaml import YAML

        yaml = YAML()
        data = yaml.map()
        data["a"] = 1
        data["b"] = 2
        with pytest.raises(TypeError):
            yaml.dump(data)

    def test_dump_too_many_args(self, tmpdir):
        from srsly.ruamel_yaml import YAML

        fn = Path(str(tmpdir)) / "test.yaml"
        yaml = YAML()
        data = yaml.map()
        data["a"] = 1
        data["b"] = 2
        with pytest.raises(TypeError):
            yaml.dump(data, fn, True)

    def test_transform(self, tmpdir):
        from srsly.ruamel_yaml import YAML

        def tr(s):
            return s.replace(" ", "  ")

        fn = Path(str(tmpdir)) / "test.yaml"
        yaml = YAML()
        data = yaml.map()
        data["a"] = 1
        data["b"] = 2
        yaml.dump(data, fn, transform=tr)
        assert fn.read_text() == "a:  1\nb:  2\n"

    def test_print(self, capsys):
        from srsly.ruamel_yaml import YAML

        yaml = YAML()
        data = yaml.map()
        data["a"] = 1
        data["b"] = 2
        yaml.dump(data, sys.stdout)
        out, err = capsys.readouterr()
        assert out == "a: 1\nb: 2\n"


class TestRead:
    def test_multi_load(self):
        # make sure reader, scanner, parser get reset
        from srsly.ruamel_yaml import YAML

        yaml = YAML()
        yaml.load("a: 1")
        yaml.load("a: 1")  # did not work in 0.15.4

    def test_parse(self):
        # ensure `parse` method is functional and can parse "unsafe" yaml
        from srsly.ruamel_yaml import YAML
        from srsly.ruamel_yaml.constructor import ConstructorError

        yaml = YAML(typ="safe")
        s = "- !User0 {age: 18, name: Anthon}"
        # should fail to load
        with pytest.raises(ConstructorError):
            yaml.load(s)
        # should parse fine
        yaml = YAML(typ="safe")
        for _ in yaml.parse(s):
            pass


class TestLoadAll:
    def test_multi_document_load(self, tmpdir):
        """this went wrong on 3.7 because of StopIteration, PR 37 and Issue 211"""
        from srsly.ruamel_yaml import YAML

        fn = Path(str(tmpdir)) / "test.yaml"
        fn.write_text(
            textwrap.dedent(
                u"""\
            ---
            - a
            ---
            - b
            ...
            """
            )
        )
        yaml = YAML()
        assert list(yaml.load_all(fn)) == [["a"], ["b"]]


class TestDuplSet:
    def test_dupl_set_00(self):
        # round-trip-loader should except
        from srsly.ruamel_yaml import YAML
        from srsly.ruamel_yaml.constructor import DuplicateKeyError

        yaml = YAML()
        with pytest.raises(DuplicateKeyError):
            yaml.load(
                textwrap.dedent(
                    """\
                !!set
                ? a
                ? b
                ? c
                ? a
                """
                )
            )


class TestDumpLoadUnicode:
    # test triggered by SamH on stackoverflow (https://stackoverflow.com/q/45281596/1307905)
    # and answer by randomir (https://stackoverflow.com/a/45281922/1307905)
    def test_write_unicode(self, tmpdir):
        from srsly.ruamel_yaml import YAML

        yaml = YAML()
        text_dict = {"text": u"HELLO_WORLD©"}
        file_name = str(tmpdir) + "/tstFile.yaml"
        yaml.dump(text_dict, open(file_name, "w", encoding="utf8", newline="\n"))
        assert open(file_name, "rb").read().decode("utf-8") == u"text: HELLO_WORLD©\n"

    def test_read_unicode(self, tmpdir):
        from srsly.ruamel_yaml import YAML

        yaml = YAML()
        file_name = str(tmpdir) + "/tstFile.yaml"
        with open(file_name, "wb") as fp:
            fp.write(u"text: HELLO_WORLD©\n".encode("utf-8"))
        text_dict = yaml.load(open(file_name, "r", encoding="utf8"))
        assert text_dict["text"] == u"HELLO_WORLD©"


class TestFlowStyle:
    def test_flow_style(self, capsys):
        # https://stackoverflow.com/questions/45791712/
        from srsly.ruamel_yaml import YAML

        yaml = YAML()
        yaml.default_flow_style = None
        data = yaml.map()
        data["b"] = 1
        data["a"] = [[1, 2], [3, 4]]
        yaml.dump(data, sys.stdout)
        out, err = capsys.readouterr()
        assert out == "b: 1\na:\n- [1, 2]\n- [3, 4]\n"


class TestOldAPI:
    @pytest.mark.skipif(sys.version_info >= (3, 0), reason="ok on Py3")
    def test_duplicate_keys_02(self):
        # Issue 165 unicode keys in error/warning
        from srsly.ruamel_yaml import safe_load
        from srsly.ruamel_yaml.constructor import DuplicateKeyError

        with pytest.raises(DuplicateKeyError):
            safe_load("type: Doméstica\ntype: International")



import gc

from . import msgpack
from .util import force_path, FilePath, JSONInputBin, JSONOutputBin


def msgpack_dumps(data: JSONInputBin) -> bytes:
    """Serialize an object to a msgpack byte string.

    data: The data to serialize.
    RETURNS (bytes): The serialized bytes.
    """
    return msgpack.dumps(data, use_bin_type=True)


def msgpack_loads(data: bytes, use_list: bool = True) -> JSONOutputBin:
    """Deserialize msgpack bytes to a Python object.

    data (bytes): The data to deserialize.
    use_list (bool): Don't use tuples instead of lists. Can make
        deserialization slower.
    RETURNS: The deserialized Python object.
    """
    # msgpack-python docs suggest disabling gc before unpacking large messages
    gc.disable()
    msg = msgpack.loads(data, raw=False, use_list=use_list)
    gc.enable()
    return msg


def write_msgpack(path: FilePath, data: JSONInputBin) -> None:
    """Create a msgpack file and dump contents.

    location (FilePath): The file path.
    data (JSONInputBin): The data to serialize.
    """
    file_path = force_path(path, require_exists=False)
    with file_path.open("wb") as f:
        msgpack.dump(data, f, use_bin_type=True)


def read_msgpack(path: FilePath, use_list: bool = True) -> JSONOutputBin:
    """Load a msgpack file.

    location (FilePath): The file path.
    use_list (bool): Don't use tuples instead of lists. Can make
        deserialization slower.
    RETURNS (JSONOutputBin): The loaded and deserialized content.
    """
    file_path = force_path(path)
    with file_path.open("rb") as f:
        # msgpack-python docs suggest disabling gc before unpacking large messages
        gc.disable()
        msg = msgpack.load(f, raw=False, use_list=use_list)
        gc.enable()
        return msg



# coding: utf-8

import pytest  # NOQA

from .roundtrip import round_trip, dedent, round_trip_load, round_trip_dump  # NOQA


def load(s):
    return round_trip_load(dedent(s))


class TestLineCol:
    def test_item_00(self):
        data = load(
            """
            - a
            - e
            - [b, d]
            - c
            """
        )
        assert data[2].lc.line == 2
        assert data[2].lc.col == 2

    def test_item_01(self):
        data = load(
            """
            - a
            - e
            - {x: 3}
            - c
            """
        )
        assert data[2].lc.line == 2
        assert data[2].lc.col == 2

    def test_item_02(self):
        data = load(
            """
            - a
            - e
            - !!set {x, y}
            - c
            """
        )
        assert data[2].lc.line == 2
        assert data[2].lc.col == 2

    def test_item_03(self):
        data = load(
            """
            - a
            - e
            - !!omap
              - x: 1
              - y: 3
            - c
            """
        )
        assert data[2].lc.line == 2
        assert data[2].lc.col == 2

    def test_item_04(self):
        data = load(
            """
         # testing line and column based on SO
         # http://stackoverflow.com/questions/13319067/
         - key1: item 1
           key2: item 2
         - key3: another item 1
           key4: another item 2
            """
        )
        assert data[0].lc.line == 2
        assert data[0].lc.col == 2
        assert data[1].lc.line == 4
        assert data[1].lc.col == 2

    def test_pos_mapping(self):
        data = load(
            """
        a: 1
        b: 2
        c: 3
        # comment
        klm: 42
        d: 4
        """
        )
        assert data.lc.key("klm") == (4, 0)
        assert data.lc.value("klm") == (4, 5)

    def test_pos_sequence(self):
        data = load(
            """
        - a
        - b
        - c
        # next one!
        - klm
        - d
        """
        )
        assert data.lc.item(3) == (4, 2)



# coding: utf-8

import re
import pytest  # NOQA

from .roundtrip import dedent


# from PyYAML docs
class Dice(tuple):
    def __new__(cls, a, b):
        return tuple.__new__(cls, [a, b])

    def __repr__(self):
        return "Dice(%s,%s)" % self


def dice_constructor(loader, node):
    value = loader.construct_scalar(node)
    a, b = map(int, value.split("d"))
    return Dice(a, b)


def dice_representer(dumper, data):
    return dumper.represent_scalar(u"!dice", u"{}d{}".format(*data))


def test_dice_constructor():
    import srsly.ruamel_yaml  # NOQA

    srsly.ruamel_yaml.add_constructor(u"!dice", dice_constructor)
    with pytest.raises(ValueError):
        data = srsly.ruamel_yaml.load(
            "initial hit points: !dice 8d4", Loader=srsly.ruamel_yaml.Loader
        )
        assert str(data) == "{'initial hit points': Dice(8,4)}"


def test_dice_constructor_with_loader():
    import srsly.ruamel_yaml  # NOQA

    with pytest.raises(ValueError):
        srsly.ruamel_yaml.add_constructor(
            u"!dice", dice_constructor, Loader=srsly.ruamel_yaml.Loader
        )
        data = srsly.ruamel_yaml.load(
            "initial hit points: !dice 8d4", Loader=srsly.ruamel_yaml.Loader
        )
        assert str(data) == "{'initial hit points': Dice(8,4)}"


def test_dice_representer():
    import srsly.ruamel_yaml  # NOQA

    srsly.ruamel_yaml.add_representer(Dice, dice_representer)
    # srsly.ruamel_yaml 0.15.8+ no longer forces quotes tagged scalars
    assert (
        srsly.ruamel_yaml.dump(dict(gold=Dice(10, 6)), default_flow_style=False)
        == "gold: !dice 10d6\n"
    )


def test_dice_implicit_resolver():
    import srsly.ruamel_yaml  # NOQA

    pattern = re.compile(r"^\d+d\d+$")
    with pytest.raises(ValueError):
        srsly.ruamel_yaml.add_implicit_resolver(u"!dice", pattern)
        assert (
            srsly.ruamel_yaml.dump(dict(treasure=Dice(10, 20)), default_flow_style=False)
            == "treasure: 10d20\n"
        )
        assert srsly.ruamel_yaml.load(
            "damage: 5d10", Loader=srsly.ruamel_yaml.Loader
        ) == dict(damage=Dice(5, 10))


class Obj1(dict):
    def __init__(self, suffix):
        self._suffix = suffix
        self._node = None

    def add_node(self, n):
        self._node = n

    def __repr__(self):
        return "Obj1(%s->%s)" % (self._suffix, self.items())

    def dump(self):
        return repr(self._node)


class YAMLObj1(object):
    yaml_tag = u"!obj:"

    @classmethod
    def from_yaml(cls, loader, suffix, node):
        import srsly.ruamel_yaml  # NOQA

        obj1 = Obj1(suffix)
        if isinstance(node, srsly.ruamel_yaml.MappingNode):
            obj1.add_node(loader.construct_mapping(node))
        else:
            raise NotImplementedError
        return obj1

    @classmethod
    def to_yaml(cls, dumper, data):
        return dumper.represent_scalar(cls.yaml_tag + data._suffix, data.dump())


def test_yaml_obj():
    import srsly.ruamel_yaml  # NOQA

    srsly.ruamel_yaml.add_representer(Obj1, YAMLObj1.to_yaml)
    srsly.ruamel_yaml.add_multi_constructor(YAMLObj1.yaml_tag, YAMLObj1.from_yaml)
    with pytest.raises(ValueError):
        x = srsly.ruamel_yaml.load("!obj:x.2\na: 1", Loader=srsly.ruamel_yaml.Loader)
        print(x)
        assert srsly.ruamel_yaml.dump(x) == """!obj:x.2 "{'a': 1}"\n"""


def test_yaml_obj_with_loader_and_dumper():
    import srsly.ruamel_yaml  # NOQA

    srsly.ruamel_yaml.add_representer(
        Obj1, YAMLObj1.to_yaml, Dumper=srsly.ruamel_yaml.Dumper
    )
    srsly.ruamel_yaml.add_multi_constructor(
        YAMLObj1.yaml_tag, YAMLObj1.from_yaml, Loader=srsly.ruamel_yaml.Loader
    )
    with pytest.raises(ValueError):
        x = srsly.ruamel_yaml.load("!obj:x.2\na: 1", Loader=srsly.ruamel_yaml.Loader)
        # x = srsly.ruamel_yaml.load('!obj:x.2\na: 1')
        print(x)
        assert srsly.ruamel_yaml.dump(x) == """!obj:x.2 "{'a': 1}"\n"""


# ToDo use nullege to search add_multi_representer and add_path_resolver
# and add some test code

# Issue 127 reported by Tommy Wang


def test_issue_127():
    import srsly.ruamel_yaml  # NOQA

    class Ref(srsly.ruamel_yaml.YAMLObject):
        yaml_constructor = srsly.ruamel_yaml.RoundTripConstructor
        yaml_representer = srsly.ruamel_yaml.RoundTripRepresenter
        yaml_tag = u"!Ref"

        def __init__(self, logical_id):
            self.logical_id = logical_id

        @classmethod
        def from_yaml(cls, loader, node):
            return cls(loader.construct_scalar(node))

        @classmethod
        def to_yaml(cls, dumper, data):
            if isinstance(data.logical_id, srsly.ruamel_yaml.scalarstring.ScalarString):
                style = data.logical_id.style  # srsly.ruamel_yaml>0.15.8
            else:
                style = None
            return dumper.represent_scalar(cls.yaml_tag, data.logical_id, style=style)

    document = dedent(
        """\
    AList:
      - !Ref One
      - !Ref 'Two'
      - !Ref
        Two and a half
    BList: [!Ref Three, !Ref "Four"]
    CList:
      - Five Six
      - 'Seven Eight'
    """
    )
    data = srsly.ruamel_yaml.round_trip_load(document, preserve_quotes=True)
    assert srsly.ruamel_yaml.round_trip_dump(
        data, indent=4, block_seq_indent=2
    ) == document.replace("\n    Two and", " Two and")



#!/usr/bin/env python
import sys
from distutils.command.build_ext import build_ext
from distutils.sysconfig import get_python_inc
from setuptools import Extension, setup, find_packages
from pathlib import Path
from Cython.Build import cythonize
from Cython.Compiler import Options
import contextlib
import os


# Preserve `__doc__` on functions and classes
# http://docs.cython.org/en/latest/src/userguide/source_files_and_compilation.html#compiler-options
Options.docstrings = True


PACKAGE_DATA = {"": ["*.pyx", "*.pxd", "*.c", "*.h"]}
PACKAGES = find_packages()
MOD_NAMES = ["srsly.msgpack._unpacker", "srsly.msgpack._packer"]
COMPILE_OPTIONS = {
    "msvc": ["/Ox", "/EHsc"],
    "mingw32": ["-O2", "-Wno-strict-prototypes", "-Wno-unused-function"],
    "other": ["-O2", "-Wno-strict-prototypes", "-Wno-unused-function"],
}
COMPILER_DIRECTIVES = {
    "language_level": -3,
    "embedsignature": True,
    "annotation_typing": False,
}
LINK_OPTIONS = {"msvc": [], "mingw32": [], "other": ["-lstdc++", "-lm"]}

if sys.byteorder == "big":
    macros = [("__BIG_ENDIAN__", "1")]
else:
    macros = [("__LITTLE_ENDIAN__", "1")]


# By subclassing build_extensions we have the actual compiler that will be used
# which is really known only after finalize_options
# http://stackoverflow.com/questions/724664/python-distutils-how-to-get-a-compiler-that-is-going-to-be-used
class build_ext_options:
    def build_options(self):
        if hasattr(self.compiler, "initialize"):
            self.compiler.initialize()
        self.compiler.platform = sys.platform[:6]
        for e in self.extensions:
            e.extra_compile_args += COMPILE_OPTIONS.get(
                self.compiler.compiler_type, COMPILE_OPTIONS["other"]
            )
            e.extra_link_args += LINK_OPTIONS.get(
                self.compiler.compiler_type, LINK_OPTIONS["other"]
            )


class build_ext_subclass(build_ext, build_ext_options):
    def build_extensions(self):
        build_ext_options.build_options(self)
        build_ext.build_extensions(self)


def clean(path):
    n_cleaned = 0
    for name in MOD_NAMES:
        name = name.replace(".", "/")
        for ext in ["so", "html", "cpp", "c"]:
            file_path = path / f"{name}.{ext}"
            if file_path.exists():
                file_path.unlink()
                n_cleaned += 1
    print(f"Cleaned {n_cleaned} files")


@contextlib.contextmanager
def chdir(new_dir):
    old_dir = os.getcwd()
    try:
        os.chdir(new_dir)
        sys.path.insert(0, new_dir)
        yield
    finally:
        del sys.path[0]
        os.chdir(old_dir)


def setup_package():
    root = Path(__file__).parent

    if len(sys.argv) > 1 and sys.argv[1] == "clean":
        return clean(root)

    with (root / "srsly" / "about.py").open("r") as f:
        about = {}
        exec(f.read(), about)

    with chdir(root):
        include_dirs = [get_python_inc(plat_specific=True), "."]
        ext_modules = []
        for name in MOD_NAMES:
            mod_path = name.replace(".", "/") + ".pyx"
            ext_modules.append(
                Extension(
                    name,
                    [mod_path],
                    language="c++",
                    include_dirs=include_dirs,
                    define_macros=macros,
                )
            )
        ext_modules.append(
            Extension(
                "srsly.ujson.ujson",
                sources=[
                    "./srsly/ujson/ujson.c",
                    "./srsly/ujson/objToJSON.c",
                    "./srsly/ujson/JSONtoObj.c",
                    "./srsly/ujson/lib/ultrajsonenc.c",
                    "./srsly/ujson/lib/ultrajsondec.c",
                ],
                include_dirs=["./srsly/ujson", "./srsly/ujson/lib"],
                extra_compile_args=["-D_GNU_SOURCE"],
            )
        )
        print("Cythonizing sources")
        ext_modules = cythonize(ext_modules, compiler_directives=COMPILER_DIRECTIVES)

        setup(
            name="srsly",
            packages=PACKAGES,
            version=about["__version__"],
            ext_modules=ext_modules,
            cmdclass={"build_ext": build_ext_subclass},
            package_data=PACKAGE_DATA,
        )


if __name__ == "__main__":
    setup_package()
